@@ -1,6 +1,7 @@
 import os
 import pathlib
 import pkg_resources
+import shutil
 import signal
 import sys
 import traceback as tb
@@ -13,7 +14,8 @@ import requests_toolbelt
 from PyQt5 import uic, QtCore, QtGui, QtWidgets
 
 from ..api import APIKeyError
-from ..dbmodel import APIModel
+from ..common import ConnectionTimeoutErrors
+from ..dbmodel import APIInterrogator, DBExtract
 from .._version import version as __version__
 
 from .api import get_ckan_api
@@ -28,55 +30,6 @@ QtGui.QIcon.setThemeSearchPaths([
 QtGui.QIcon.setThemeName(".")
 
 
-class StatusWidget(QtWidgets.QWidget):
-    clicked = QtCore.pyqtSignal()
-
-    def __init__(self, *args, **kwargs):
-        super(StatusWidget, self).__init__(*args, **kwargs)
-        self.layout = QtWidgets.QHBoxLayout(self)
-        self.layout.setContentsMargins(0, 0, 0, 0)
-        self.layout.setSpacing(0)
-
-        self.flabel = QtWidgets.QLabel(self)
-        self.layout.addWidget(self.flabel)
-
-        self.toolButton_user = QtWidgets.QToolButton()
-        self.toolButton_user.setText("Initialization...")
-        self.toolButton_user.setToolButtonStyle(
-            QtCore.Qt.ToolButtonTextBesideIcon)
-        self.toolButton_user.setAutoRaise(True)
-        self.layout.addWidget(self.toolButton_user)
-        self.toolButton_user.clicked.connect(self.clicked)
-
-    def get_favicon(self, server):
-        dldir = pathlib.Path(appdirs.user_cache_dir(appname="DCOR-Aid",
-                                                    appauthor="DCOR"))
-        dldir.mkdir(exist_ok=True, parents=True)
-        favname = dldir / (server.split("://")[1] + "_favicon.ico")
-        if not favname.exists():
-            try:
-                r = requests.get(server + "/favicon.ico")
-                if r.ok:
-                    with favname.open("wb") as fd:
-                        fd.write(r.content)
-                else:
-                    raise ValueError("No favicon!")
-                favicon = QtGui.QIcon(str(favname))
-            except BaseException:
-                favicon = QtGui.QIcon()
-        else:
-            favicon = QtGui.QIcon(str(favname))
-        return favicon
-
-    def set_status(self, text, tooltip, icon, server):
-        favicon = self.get_favicon(server)
-        self.flabel.setPixmap(favicon.pixmap(16, 16))
-        self.flabel.setToolTip(server)
-        self.toolButton_user.setText(text)
-        self.toolButton_user.setToolTip(tooltip)
-        self.toolButton_user.setIcon(QtGui.QIcon.fromTheme(icon))
-
-
 class DCORAid(QtWidgets.QMainWindow):
     plots_changed = QtCore.pyqtSignal()
 
@@ -87,6 +40,7 @@ class DCORAid(QtWidgets.QMainWindow):
         application will print the version after initialization
         and exit.
         """
+        super(DCORAid, self).__init__(*args, **kwargs)
         # Settings are stored in the .ini file format. Even though
         # `self.settings` may return integer/bool in the same session,
         # in the next session, it will reliably return strings. Lists
@@ -97,9 +51,21 @@ class DCORAid(QtWidgets.QMainWindow):
         QtCore.QCoreApplication.setOrganizationDomain("dcor.mpl.mpg.de")
         QtCore.QCoreApplication.setApplicationName("dcoraid")
         QtCore.QSettings.setDefaultFormat(QtCore.QSettings.IniFormat)
+
+        # TODO: Remove this block in a future release
+        # BEGIN REMOVE
+        # Clear the old cache. Unfortunately, it does not make sense to
+        # move the cache to a new location, because the old version
+        # of dclab did not use temporary files when compressing data.
+        # Hence, we cannot be sure that the compression task successfully
+        # finished.
+        old_cache = pathlib.Path(appdirs.user_cache_dir()) / "dcoraid"
+        if old_cache.exists():
+            shutil.rmtree(old_cache, ignore_errors=True)
+        # END REMOVE
+
         #: DCOR-Aid settings
         self.settings = QtCore.QSettings()
-        super(DCORAid, self).__init__(*args, **kwargs)
         path_ui = pkg_resources.resource_filename(
             "dcoraid.gui", "main.ui")
         uic.loadUi(path_ui, self)
@@ -113,7 +79,7 @@ class DCORAid(QtWidgets.QMainWindow):
         # Preferences dialog
         self.dlg_pref = PreferencesDialog()
         # Window title
-        self.setWindowTitle("DCOR-Aid {}".format(__version__))
+        self.setWindowTitle(f"DCOR-Aid {__version__}")
         # Disable native menubar (e.g. on Mac)
         self.menubar.setNativeMenuBar(False)
         # File menu
@@ -122,24 +88,37 @@ class DCORAid(QtWidgets.QMainWindow):
         # Help menu
         self.actionSoftware.triggered.connect(self.on_action_software)
         self.actionAbout.triggered.connect(self.on_action_about)
+
         # Display login status
         self.status_widget = StatusWidget(self)
         self.tabWidget.setCornerWidget(self.status_widget)
         self.status_widget.clicked.connect(self.dlg_pref.on_show_server)
-        self.refresh_login_status()
         # Call refresh_login status regularly
-        self.timer = QtCore.QTimer()
-        self.timer.timeout.connect(self.refresh_login_status)
-        self.timer.start(300000)
-        # Update private data tab
-        self.refresh_private_data()
-        # If a new dataset has been uploaded, refresh private data
-        self.panel_upload.upload_finished.connect(self.refresh_private_data)
+        self.refresh_login_status()  # runs asynchronously
+        if bool(int(self.settings.value("debug/without timers", "0"))):
+            self.timer = None
+        else:
+            self.timer = QtCore.QTimer()
+            self.timer.timeout.connect(self.refresh_login_status)
+            self.timer.start(300000)
 
+        # Signals for user datasets (my data)
+        self.pushButton_user_refresh.clicked.connect(
+            self.on_refresh_private_data)
+
+        # Signals for public data browser
+        self.pushButton_public_search.clicked.connect(self.on_public_search)
+
+        # Signal for requesting resource download
+        self.public_filter_chain.download_resource.connect(
+            self.panel_download.download_resource)
+
+        QtWidgets.QApplication.processEvents(QtCore.QEventLoop.AllEvents,
+                                             300)
+
+        # Run wizard if necessary
         if ((self.settings.value("user scenario", "") != "anonymous")
                 and not self.settings.value("auth/api key", "")):
-            QtWidgets.QApplication.processEvents(QtCore.QEventLoop.AllEvents,
-                                                 300)
             # User has not done anything yet
             self.on_wizard()
 
@@ -147,8 +126,10 @@ class DCORAid(QtWidgets.QMainWindow):
         self.raise_()
 
     def close(self):
-        self.timer.stop()
-        self.panel_upload.widget_jobs.timer.stop()
+        if self.timer is not None:
+            self.timer.stop()
+        if self.panel_upload.widget_jobs.timer is not None:
+            self.panel_upload.widget_jobs.timer.stop()
         self.panel_upload.jobs.daemon_compress.join()
         self.panel_upload.jobs.daemon_upload.join()
         self.panel_upload.jobs.daemon_verify.join()
@@ -175,8 +156,7 @@ class DCORAid(QtWidgets.QMainWindow):
                                     about_text)
 
     def on_action_software(self):
-        libs = [appdirs,
-                dclab,
+        libs = [dclab,
                 requests,
                 requests_toolbelt,
                 ]
@@ -194,6 +174,44 @@ class DCORAid(QtWidgets.QMainWindow):
                                           "Software",
                                           sw_text)
 
+    @QtCore.pyqtSlot()
+    def on_public_search(self):
+        self.tab_browse.setCursor(QtCore.Qt.WaitCursor)
+        api = get_ckan_api(
+            public=not self.checkBox_public_include_private.isChecked())
+        try:
+            ai = APIInterrogator(api=api)
+            dbextract = ai.search_dataset(
+                self.lineEdit_public_search.text(),
+                limit=self.spinBox_public_rows.value())
+            self.public_filter_chain.set_db_extract(dbextract)
+        except ConnectionTimeoutErrors:
+            QtWidgets.QMessageBox.critical(
+                self,
+                f"Failed to connect to {api.server}",
+                tb.format_exc(limit=1))
+        self.tab_browse.setCursor(QtCore.Qt.ArrowCursor)
+
+    @QtCore.pyqtSlot()
+    def on_refresh_private_data(self):
+        self.tab_user.setCursor(QtCore.Qt.WaitCursor)
+        api = get_ckan_api()
+        data = DBExtract()
+        if api.is_available() and api.api_key:
+            try:
+                ai = APIInterrogator(api=api)
+                if self.checkBox_user_following.isChecked():
+                    data += ai.get_datasets_user_following()
+                if self.checkBox_user_owned.isChecked():
+                    data += ai.get_datasets_user_owned()
+                self.user_filter_chain.set_db_extract(data)
+            except ConnectionTimeoutErrors:
+                QtWidgets.QMessageBox.critical(
+                    self,
+                    f"Failed to connect to {api.server}",
+                    tb.format_exc(limit=1))
+        self.tab_user.setCursor(QtCore.Qt.ArrowCursor)
+
     @QtCore.pyqtSlot()
     def on_wizard(self):
         self.wizard = SetupWizard(self)
@@ -203,51 +221,85 @@ class DCORAid(QtWidgets.QMainWindow):
     @QtCore.pyqtSlot()
     def refresh_login_status(self):
         api = get_ckan_api()
-        if not api.is_available():
-            text = "No connection"
-            tip = "Can you access {} via a browser?".format(api.server)
-            icon = "hourglass"
+        if not api.api_key:
+            text = "Anonymous"
+            tip = "Click here to enter your API key."
+            icon = "user"
         else:
-            if not api.api_key:
-                text = "Anonymous"
-                tip = "Click here to enter your API key."
-                icon = "user"
+            try:
+                user_data = api.get_user_dict()
+            except APIKeyError:
+                text = "Login failed"
+                tip = "Click here to update your API key."
+                icon = "user-times"
+            except ConnectionTimeoutErrors:
+                text = "No connection"
+                tip = "Can you access {} via a browser?".format(api.server)
+                icon = "hourglass"
             else:
-                try:
-                    user_data = api.get_user_dict()
-                except APIKeyError:
-                    text = "Login failed"
-                    tip = "Click here to update your API key."
-                    icon = "user-times"
-                else:
-                    fullname = user_data["fullname"]
-                    name = user_data["name"]
-                    if not fullname:
-                        fullname = name
-                    text = "{}".format(fullname)
-                    tip = "user '{}'".format(name)
-                    icon = "user-lock"
+                fullname = user_data["fullname"]
+                name = user_data["name"]
+                if not fullname:
+                    fullname = name
+                text = "{}".format(fullname)
+                tip = "user '{}'".format(name)
+                icon = "user-lock"
         self.status_widget.set_status(text=text,
                                       tooltip=tip,
                                       icon=icon,
                                       server=api.server)
 
-    @run_async
-    @QtCore.pyqtSlot()
-    def refresh_private_data(self):
-        self.tab_user.setCursor(QtCore.Qt.WaitCursor)
-        # TODO:
-        # - what happens if the user changes the server? Ask to restart?
-        api = get_ckan_api()
-        try:
-            am = APIModel(api=api)
-            if am.api.is_available():
-                db_extract = am.get_user_datasets()
-        except BaseException:
-            pass
+
+class StatusWidget(QtWidgets.QWidget):
+    clicked = QtCore.pyqtSignal()
+
+    def __init__(self, *args, **kwargs):
+        super(StatusWidget, self).__init__(*args, **kwargs)
+        self.layout = QtWidgets.QHBoxLayout(self)
+        self.layout.setContentsMargins(0, 0, 0, 0)
+        self.layout.setSpacing(0)
+
+        self.flabel = QtWidgets.QLabel(self)
+        self.layout.addWidget(self.flabel)
+
+        self.toolButton_user = QtWidgets.QToolButton()
+        self.toolButton_user.setText("Initialization...")
+        self.toolButton_user.setToolButtonStyle(
+            QtCore.Qt.ToolButtonTextBesideIcon)
+        self.toolButton_user.setAutoRaise(True)
+        self.layout.addWidget(self.toolButton_user)
+        self.toolButton_user.clicked.connect(self.clicked)
+
+    @staticmethod
+    def get_favicon(server):
+        dldir = pathlib.Path(
+            QtCore.QStandardPaths.writableLocation(
+                QtCore.QStandardPaths.AppDataLocation)) / "favicons"
+
+        dldir.mkdir(exist_ok=True, parents=True)
+        favname = dldir / (server.split("://")[1] + "_favicon.ico")
+        if not favname.exists():
+            try:
+                r = requests.get(server + "/favicon.ico", timeout=3.05)
+                if r.ok:
+                    with favname.open("wb") as fd:
+                        fd.write(r.content)
+                else:
+                    raise ValueError("No favicon!")
+                favicon = QtGui.QIcon(str(favname))
+            except BaseException:
+                favicon = QtGui.QIcon()
         else:
-            self.user_filter_chain.set_db_extract(db_extract)
-        self.tab_user.setCursor(QtCore.Qt.ArrowCursor)
+            favicon = QtGui.QIcon(str(favname))
+        return favicon
+
+    def set_status(self, text, tooltip, icon, server):
+        favicon = self.get_favicon(server)
+        self.flabel.setPixmap(favicon.pixmap(16, 16))
+        self.flabel.setToolTip(server)
+        self.toolButton_user.setText(text)
+        self.toolButton_user.setToolTip(tooltip)
+        self.toolButton_user.setIcon(QtGui.QIcon.fromTheme(icon))
 
 
 def excepthook(etype, value, trace):
