@@ -1,18 +1,25 @@
-from functools import lru_cache
 import pathlib
 import pkg_resources
 import traceback as tb
 
 from PyQt5 import uic, QtCore, QtGui, QtWidgets
 
-from ...upload import create_dataset
 
+from ...api import dataset_create
+
+from ..tools import ShowWaitCursor
 from ..api import get_ckan_api
 
+from . import circle_mgr
 from .resources_model import ResourcesModel
 from .resource_schema_preset import PersistentResourceSchemaPresets
 
 
+class NoCircleSelectedError(ValueError):
+    """Used to tell UploadWidget that the user did not select a circle"""
+    pass
+
+
 class UploadDialog(QtWidgets.QDialog):
     finished = QtCore.pyqtSignal(object)
     instance_counter = 1
@@ -45,23 +52,24 @@ class UploadDialog(QtWidgets.QDialog):
         # Initialize api
         self.api = get_ckan_api()
 
-        # Set license choices
-        licenses = self.api.get_license_list()
-        for lic in licenses:
-            if lic["domain_data"]:  # just some identifier to exclude "none"
-                self.comboBox_license.addItem(
-                    "{} ({})".format(lic["title"], lic["id"]), lic["id"])
-
-        # Set supplementary resource schema
-        rss = self.api.get_supplementary_resource_schema()
-        self.widget_schema.populate_schema(rss)
-
         # Set circle choices
         circles = self.get_user_circle_dicts()
         for ci in circles:
             self.comboBox_circles.addItem(
                 ci["title"] if ci["title"] else ci["name"], ci["name"])
 
+        with ShowWaitCursor():
+            # Set license choices
+            licenses = self.api.get_license_list()
+            for lic in licenses:
+                if lic["domain_data"]:  # just a identifier to exclude "none"
+                    self.comboBox_license.addItem(
+                        "{} ({})".format(lic["title"], lic["id"]), lic["id"])
+
+            # Set supplementary resource schema
+            rss = self.api.get_supplementary_resource_schema()
+            self.widget_schema.populate_schema(rss)
+
         # Set visibility choices
         settings = QtCore.QSettings()
         if settings.value("user scenario", "") == "medical":
@@ -114,6 +122,9 @@ class UploadDialog(QtWidgets.QDialog):
         self.toolButton_preset_load.clicked.connect(self.on_preset_load)
         self.toolButton_preset_store.clicked.connect(self.on_preset_store)
 
+        # Do not allow drag and drop to line edit of combobox
+        self.comboBox_preset.lineEdit().setAcceptDrops(False)
+
     def _autofill_for_testing(self, **kwargs):
         self.lineEdit_title.setText(kwargs.get("title", "Dataset Title"))
         self.lineEdit_authors.setText(kwargs.get("authors", "John Doe"))
@@ -132,35 +143,36 @@ class UploadDialog(QtWidgets.QDialog):
         if path.exists():
             self.on_add_resources([str(path.resolve())])
 
-    @lru_cache(maxsize=1)
+    def dragEnterEvent(self, e):
+        """Whether files are accepted"""
+        if e.mimeData().hasUrls():
+            e.accept()
+        else:
+            e.ignore()
+
+    def dropEvent(self, e):
+        """Add dropped files to view"""
+        urls = e.mimeData().urls()
+        pathlist = []
+        for ff in urls:
+            pp = pathlib.Path(ff.toLocalFile())
+            if pp.is_dir():
+                pathlist += list(pp.rglob("*.rtdc"))
+            else:
+                pathlist.append(pp)
+        if pathlist:
+            self.on_add_resources(files=sorted(pathlist))
+
     def get_user_circle_dicts(self):
-        circles = self.api.get("organization_list_for_user",
-                               permission="create_dataset")
+        with ShowWaitCursor():
+            circles = circle_mgr.get_user_circle_dicts()
         if not circles:
-            # Ask the user whether he would like to create a circle.
-            ud = self.api.get_user_dict()
-            name = ud["fullname"] if ud["fullname"] else ud["name"]
-            text, okPressed = QtWidgets.QInputDialog.getText(
-                self,
-                "Circle required",
-                "You do not have access to any existing Circles. To upload\n"
-                + "datasets, you need to be either Editor or Admin in\n"
-                + "a Circle. You may create a Circle now or cancel and ask\n"
-                + "a colleague to add you to a Circle (Your user name is "
-                + "'{}').".format(ud["name"])
-                + "\n\nTo proceed with Circle creation, please choose a name:",
-                QtWidgets.QLineEdit.Normal,
-                "{}'s Circle".format(name))
-            if okPressed and text != '':
-                cname = "user-circle-{}".format(ud["name"])
-                cdict = self.api.post("organization_create",
-                                      data={"name": cname,
-                                            "title": text.strip(),
-                                            })
+            cdict = circle_mgr.ask_for_new_circle(self)
+            if cdict:
                 circles.append(cdict)
             else:
-                self.deleteLater()
-                self.close()
+                # abort here
+                raise NoCircleSelectedError("User did not specify a circle!")
         return circles
 
     def assemble_metadata(self):
@@ -205,8 +217,8 @@ class UploadDialog(QtWidgets.QDialog):
                 self, "Resources to upload", ".",
                 "Supported file types ({})".format(
                     " ".join(["*{}".format(s) for s in suffixes])))
+        files = [str(ff) for ff in files]  # make sure files are strings
         self.rvmodel.add_resources(files)
-
         if not self.listView_resources.selectedIndexes():
             # Select the first item
             ix = self.rvmodel.index(0, 0)
@@ -314,7 +326,7 @@ class UploadDialog(QtWidgets.QDialog):
 
         # Try to create the dataset and display any issues with the metadata
         try:
-            data = create_dataset(dataset_dict=self.assemble_metadata(),
+            data = dataset_create(dataset_dict=self.assemble_metadata(),
                                   api=self.api.copy())
         except BaseException:
             msg = QtWidgets.QMessageBox()
